@@ -1,22 +1,26 @@
-#![deny(bare_trait_objects, anonymous_parameters, elided_lifetimes_in_paths)]
+//! cargo-outdated
+//! A subcommand for cargo that checks if your dependencies are up-to-date
 
-use cargo;
-use env_logger;
+#![deny(bare_trait_objects, anonymous_parameters, elided_lifetimes_in_paths)]
 
 #[macro_use]
 mod macros;
 mod cargo_ops;
-use crate::cargo_ops::{ElaborateWorkspace, TempProject};
+mod error;
+
+use crate::{
+    cargo_ops::{ElaborateWorkspace, TempProject},
+    error::OutdatedError,
+};
 
-use cargo::core::maybe_allow_nightly_features;
 use cargo::core::shell::Verbosity;
 use cargo::core::Workspace;
 use cargo::ops::needs_custom_http_transport;
 use cargo::util::important_paths::find_root_manifest_for_wd;
 use cargo::util::{CargoResult, CliError, Config};
 use docopt::Docopt;
-use git2_curl;
 
+/// usage message for --help
 pub const USAGE: &str = "
 Displays information about project dependency versions
 
@@ -28,6 +32,8 @@ Options:
     -h, --help                  Prints help information
         --format FORMAT         Output formatting [default: list]
                                 [values: list, json]
+    -i, --ignore DEPENDENCIES   Comma separated list of dependencies to not print in the output
+    -x, --exclude DEPENDENCIES  Comma separated list of dependencies to exclude from building
     -q, --quiet                 Suppresses warnings
     -R, --root-deps-only        Only check root dependencies (Equivalent to --depth=1)
     -V, --version               Prints version information
@@ -44,6 +50,7 @@ Options:
                                 (Defaults to Cargo.toml in project root)
     -p, --packages PKGS         Packages to inspect for updates
     -r, --root ROOT             Package to treat as the root package
+    -o, --offline               Run without accessing the network (useful for testing w/ local registries)
 ";
 
 /// Options from CLI arguments
@@ -52,6 +59,8 @@ pub struct Options {
     flag_format: Option<String>,
     flag_color: Option<String>,
     flag_features: Vec<String>,
+    flag_ignore: Vec<String>,
+    flag_exclude: Vec<String>,
     flag_manifest_path: Option<String>,
     flag_quiet: bool,
     flag_verbose: u32,
@@ -62,6 +71,7 @@ pub struct Options {
     flag_root_deps_only: bool,
     flag_workspace: bool,
     flag_aggressive: bool,
+    flag_offline: bool,
 }
 
 impl Options {
@@ -96,6 +106,8 @@ fn main() {
                 .collect()
         }
         options.flag_features = flat_split(&options.flag_features);
+        options.flag_ignore = flat_split(&options.flag_ignore);
+        options.flag_exclude = flat_split(&options.flag_exclude);
         options.flag_packages = flat_split(&options.flag_packages);
         if options.flag_root_deps_only {
             options.flag_depth = Some(1);
@@ -103,8 +115,6 @@ fn main() {
         options
     };
 
-
-
     let mut config = match Config::default() {
         Ok(cfg) => cfg,
         Err(e) => {
@@ -116,21 +126,15 @@ fn main() {
     // Only use a custom transport if any HTTP options are specified,
     // such as proxies or custom certificate authorities. The custom
     // transport, however, is not as well battle-tested.
-    // See cargo-outdated issue #197 and 
-    // https://github.com/rust-lang/cargo/blob/master/src/bin/cargo/main.rs#L181 
+    // See cargo-outdated issue #197 and
+    // https://github.com/rust-lang/cargo/blob/master/src/bin/cargo/main.rs#L181
     // fn init_git_transports()
-    match needs_custom_http_transport(&config) {
-        Ok(true) => {
-            match cargo::ops::http_handle(&config) {
-                Ok(handle) => {
-                    unsafe {
-                        git2_curl::register(handle);
-                    }
-                },
-                Err(_) => {}
+    if let Ok(true) = needs_custom_http_transport(&config) {
+        if let Ok(handle) = cargo::ops::http_handle(&config) {
+            unsafe {
+                git2_curl::register(handle);
             }
-        },
-        _ => {},
+        }
     }
 
     let exit_code = options.flag_exit_code;
@@ -151,30 +155,28 @@ fn main() {
     }
 }
 
+/// executes the cargo-outdate command with the cargo configuration and options
 pub fn execute(options: Options, config: &mut Config) -> CargoResult<i32> {
-    // Check if $CARGO_HOME is set before capturing the config environment 
-    // if it is, set it in the configure options 
-    let cargo_home_path = match std::env::var_os("CARGO_HOME") {
-        Some(path) => Some(std::path::PathBuf::from(path)),
-        None => None
-    };
+    // Check if $CARGO_HOME is set before capturing the config environment
+    // if it is, set it in the configure options
+    let cargo_home_path = std::env::var_os("CARGO_HOME").map(std::path::PathBuf::from);
+
+    // enabling nightly features
+    config.nightly_features_allowed = true;
 
     config.configure(
         options.flag_verbose,
-        None,
+        options.flag_quiet,
         options.flag_color.as_deref(),
         options.frozen(),
         options.locked(),
-        false,
+        options.flag_offline,
         &cargo_home_path,
         &[],
         &[],
     )?;
     debug!(config, format!("options: {:?}", options));
 
-    // Needed to allow nightly features
-    maybe_allow_nightly_features();
-
     verbose!(config, "Parsing...", "current workspace");
     // the Cargo.toml that we are actually working on
     let mut manifest_abspath: std::path::PathBuf;
@@ -212,8 +214,12 @@ pub fn execute(options: Options, config: &mut Config) -> CargoResult<i32> {
     compat_proj.cargo_update()?;
     verbose!(config, "Resolving...", "compat workspace");
     let compat_workspace = compat_proj.workspace.borrow();
-    let ela_compat =
-        ElaborateWorkspace::from_workspace(compat_workspace.as_ref().unwrap(), &options)?;
+    let ela_compat = ElaborateWorkspace::from_workspace(
+        compat_workspace
+            .as_ref()
+            .ok_or(OutdatedError::CannotElaborateWorkspace)?,
+        &options,
+    )?;
 
     verbose!(config, "Parsing...", "latest workspace");
     let latest_proj =
@@ -227,8 +233,12 @@ pub fn execute(options: Options, config: &mut Config) -> CargoResult<i32> {
     latest_proj.cargo_update()?;
     verbose!(config, "Resolving...", "latest workspace");
     let latest_workspace = latest_proj.workspace.borrow();
-    let ela_latest =
-        ElaborateWorkspace::from_workspace(latest_workspace.as_ref().unwrap(), &options)?;
+    let ela_latest = ElaborateWorkspace::from_workspace(
+        latest_workspace
+            .as_ref()
+            .ok_or(OutdatedError::CannotElaborateWorkspace)?,
+        &options,
+    )?;
 
     if ela_curr.workspace_mode {
         let mut sum = 0;
